@@ -1,3 +1,16 @@
+# Part of the varbvs package, https://github.com/pcarbo/varbvs
+#
+# Copyright (C) 2012-2017, Peter Carbonetto
+#
+# This program is free software: you can redistribute it under the
+# terms of the GNU General Public License; either version 3 of the
+# License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANY; without even the implied warranty of
+# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+# General Public License for more details.
+#
 # Summarize the variable selection results in a single plot.
 plot.varbvs <-
     function (x, score, groups, vars = NULL, var.labels, draw.threshold = NA,
@@ -20,11 +33,10 @@ plot.varbvs <-
   
   # PROCESS OPTIONS
   # ---------------
-  # Calculate the posterior inclusion probabilities (PIPs) if a
+  # Get the posterior inclusion probabilities (PIPs) if a
   # "score" isn't provided as one of the inputs.
   if (missing(score)) {
-    w <- c(normalizelogweights(x$logw))
-    y <- x$alpha %*% w
+    y <- x$pip
   } else
     y <- score
   p <- length(y)
