@@ -1,3 +1,16 @@
+// Part of the varbvs package, https://github.com/pcarbo/varbvs
+//
+// Copyright (C) 2012-2017, Peter Carbonetto
+//
+// This program is free software: you can redistribute it under the
+// terms of the GNU General Public License; either version 3 of the
+// License, or (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANY; without even the implied warranty of
+// MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+// General Public License for more details.
+//
 #ifndef INCLUDE_MISC
 #define INCLUDE_MISC
 
@@ -15,13 +28,21 @@ double sigmoid (double x);
 // performed in a numerically stable manner.
 double logsigmoid (double x);
 
+// This function takes as input an array of unnormalized
+// log-probabilities "logw" and returns normalized probabilities "w"
+// such that the sum is equal to 1.
+void normalizelogweights (const double* logw, double* w, Size n);
+
 // Copy entries of one vector to another vector.
 void copy (const double* source, double* dest, Size n);
 
 // Get a pointer to column j of matrix X. The entries in a single
 // column of the matrix are assumed to be stored consecutively in
 // memory. Input n is the number of rows in the matrix.
-const MatrixElem* getColumn (const MatrixElem* X, Index j, Size n);
+const MatrixElem* getConstColumn (const MatrixElem* X, Index j, Size n);
+
+// Get a pointer to column j of matrix X.
+MatrixElem* getColumn (MatrixElem* X, Index j, Size n);
 
 // Copy column j of matrix X. The entries in a single column of the
 // matrix are assumed to be stored consecutively in memory. Input n
